-- Resize.applescript
-- Resize

--  Created by Nico Rohrbach on 29.04.06.
--  Copyright 2006 Superpixel, Nico Rohrbach. All rights reserved.


property resize_width : 200
property file_path : missing value
property default_delimiter : missing value
property supported_types : {"png", "gif", "tif", "tiff", "jpg", "jpeg"}
property preview_image : missing value
property open_panel_path : missing value
property open_panel_directory : "~/Desktop/"
property open_panel_filetypes : {"png", "gif", "tif", "tiff", "jpg", "jpeg"}

property which_panel : missing value -- open_panel, save_panel
property save_path : missing value



(* ==== Event Handlers ==== *)


on will finish launching theObject
	my registerSettings()
	my readSettings()
	--
	set default_delimiter to AppleScript's text item delimiters
	my writetoUI()
	my setUI(false)
end will finish launching


on awake from nib theObject
	tell text field "filename" of window "main" to register drag types {"file names"}
end awake from nib


on launched theObject
	show window "main"
end launched


on clicked theObject
	
	if name of theObject is "choose" then
		my show_open_panel()
		
	else if name of theObject is "resize" then
		if readfromUI() is false then
			beep
			display alert "The entered width is invalid!" as critical message 
				"You must enter a number as positive integer." default button "OK" attached to window "main"
			return
		end if
		
		if resize_width is "" then
			beep
			display alert "No width is set!" as informational message 
				"You must set a width in pixels to resize the image." default button "OK" attached to window "main"
			return
		end if
		tell save panel
			set title to "Save As"
			set prompt to "Resize and Save"
			set required file type to "jpg"
			set treat packages as directories to false
		end tell
		set which_panel to "save_panel"
		display save panel in directory "~/Desktop" with file name "" attached to window of theObject
		
		
	else if name of theObject is "clear" then
		my setUI(false)
		
	end if
end clicked


on choose menu item theObject
	if name of theObject is "open" then
		my show_open_panel()
	end if
end choose menu item


on drop theObject drag info dragInfo
	-- Get the list of data types on the pasteboard
	set dataTypes to types of pasteboard of dragInfo
	
	-- We are only interested in "file names"
	if "file names" is in dataTypes then
		
		-- Initialize the list of files to an empty list
		set theFiles to {}
		
		-- We want the data as a list of file names, so set the preferred type to "file names"
		set preferred type of pasteboard of dragInfo to "file names"
		
		-- Get the list of files from the pasteboard
		set theFiles to contents of pasteboard of dragInfo
		
		-- Make sure we have only one item
		if (count of theFiles) > 1 then
			beep
			display alert "You droped more than one file." as informational message 
				"Resize can process files only individually." default button "OK" attached to window "main"
			set preferred type of pasteboard of dragInfo to ""
			return false
			
		else if (count of theFiles) > 0 then
			try
				set file_path to first item of theFiles
				set AppleScript's text item delimiters to "."
				set file_extension to last text item of file_path
				
				if file_extension is in supported_types then
					my prepare_file()
					return true
				else
					beep
					display alert "You droped an unsupported file type." as informational message 
						"Drop only image files with a correct file extension. Resize can handle PNG, GIF, TIFF and JPEG files." default button "OK" attached to window "main"
					return false
				end if
			on error err_msg
				return false
			end try
		end if
	end if
end drop


on panel ended theObject with result withResult
	
	if which_panel is equal to "open_panel" then
		if withResult is 1 then -- 1 is returned if the user NOT clicks cancel
			-- Convert the list into a list of strings separated by return characters that we can put in the 'path names' text view
			-- For some unknown (as of yet) you must coerce the 'path names' to a list (even though it is defined as list).
			set open_panel_path to (path names of open panel as list)
			set AppleScript's text item delimiters to return
			set file_path to open_panel_path as string
			my prepare_file()
		end if
		
	else if which_panel is equal to "save_panel" then
		if withResult is 1 then -- 1 is returned if the user NOT clicks cancel
			set save_path to (path name of save panel)
			my resize_image()
		end if
		
	end if
end panel ended


on should close theObject
	if readfromUI() is false then
		set resize_width to 200
	end if
	return true
end should close


on should quit after last window closed theObject
	return true
end should quit after last window closed


on will quit theObject
	my writeSettings()
end will quit


(* Registers the settings (application preferences) with the 'user defaults'. *)
on registerSettings()
	tell user defaults
		-- Add all of the new default entries
		make new default entry at end of default entries with properties {name:"resize_width", contents:resize_width}
		
		-- Now we need to register the new entries in the user defaults
		register
	end tell
end registerSettings


(* Reads the settings (application preferences) from the 'user defaults'. *)
on readSettings()
	tell user defaults
		set resize_width to contents of default entry "resize_width"
	end tell
end readSettings


(* Writes the settings (application preferences) to the 'user defaults'. *)
on writeSettings()
	tell user defaults
		set contents of default entry "resize_width" to resize_width
	end tell
end writeSettings




(* ==== Private Handlers ==== *)


on prepare_file()
	-- first we try to delete existing images. we don't like memory leaks!
	try
		delete preview_image
	end try
	
	try
		set preview_image to load image file_path
		set image of image view "preview" of window "main" to preview_image
	on error err_msg
		beep
		my setUI(false)
		return
	end try
	
	my setUI(true)
	
end prepare_file


on setUI(element_status)
	if element_status is true then
		set enabled of button "resize" of window "main" to true
		set visible of button "clear" of window "main" to true
		set visible of text field "nopreview" of window "main" to false
		set contents of text field "filename" of window "main" to (file_path as string)
	else if element_status is false then
		set enabled of button "resize" of window "main" to false
		set visible of button "clear" of window "main" to false
		set visible of text field "nopreview" of window "main" to true
		set contents of text field "filename" of window "main" to ""
		--
		try
			delete preview_image
		end try
		delete image of image view "preview" of window "main"
		--
		set file_path to missing value
	end if
end setUI


on show_open_panel()
	-- preparing open panel
	tell open panel
		set title to "Choose image"
		set prompt to "Choose"
		set treat packages as directories to false
		set can choose directories to false
		set can choose files to true
		set allows multiple selection to false
	end tell
	-- show the panel
	set which_panel to "open_panel"
	display open panel in directory open_panel_directory for file types open_panel_filetypes attached to window "main"
end show_open_panel


on readfromUI()
	try
		set resize_width to content of text field "width" of window "main" as integer
	on error err_msg
		return false
	end try
	return true
end readfromUI

on writetoUI()
	set content of text field "width" of window "main" to resize_width
end writetoUI


on resize_image()
	tell window "main"
		set uses threaded animation of progress indicator "spinner" to true
		tell progress indicator "spinner" to start
	end tell
	delay 0.5
	-- now do the scaling and saving work
	set the_command to "/usr/bin/sips" & space & 
		"--resampleWidth" & space & resize_width & space & 
		"--setProperty format jpeg" & space & (quoted form of file_path) & space & 
		"--out" & space & (quoted form of save_path)
	try
		do shell script the_command
	on error err_msg
		beep
		tell window "main"
			tell progress indicator "spinner" to stop
		end tell
		display alert "The image cannot be saved!" as critical message 
			(err_msg as text) default button "OK" -- attached to window "main"
		return
	end try
	tell window "main"
		tell progress indicator "spinner" to stop
	end tell
end resize_image

